@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 ### System ###
 import os
 import re
@@ -12,8 +13,17 @@ from contextlib import contextmanager
 from subprocess import run, PIPE, STDOUT, Popen
 
 ### Logging ###
-import logzero
-from logzero import logger
+from rich import print
+from rich.logging import RichHandler
+
+logging.basicConfig(
+    level=logging.INFO,
+    format="%(message)s",
+    datefmt="[%X]",
+    handlers=[RichHandler(rich_tracebacks=True, markup=True)],
+)
+
+logger = logging.getLogger("rich")
 
 ### CLI Parsing ###
 import click
@@ -29,11 +39,12 @@ from semantic_version import Version
 from tqdm import tqdm
 
 
-semver = re.compile(r"^((0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)(-(0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(\.(0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*)?(\+[0-9a-zA-Z-]+(\.[0-9a-zA-Z-]+)*)?)/?$")  # noqa: E501
+semver = re.compile(
+    r"^((0|[1-9]\d*)\.(0|[1-9]\d*)\.(0|[1-9]\d*)(-(0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)(\.(0|[1-9]\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*)?(\+[0-9a-zA-Z-]+(\.[0-9a-zA-Z-]+)*)?)/?$"
+)
 
 
 class AliasedGroup(click.Group):
-
     def get_command(self, ctx, cmd_name):
         rv = click.Group.get_command(self, ctx, cmd_name)
         if rv is not None:
@@ -46,8 +57,7 @@ class AliasedGroup(click.Group):
         ctx.fail("Too many matches: {}".format(", ".join(sorted(matches))))
 
 
-class ComparableVersion():
-
+class ComparableVersion:
     def __init__(self, version=None):
         if isinstance(version, str):
             version = Version(version)
@@ -76,7 +86,6 @@ class ComparableVersion():
 
 
 class RenpyInstance(ComparableVersion):
-
     def __init__(self, version=None, path=None):
         super(RenpyInstance, self).__init__(version)
         self.path = path
@@ -84,11 +93,12 @@ class RenpyInstance(ComparableVersion):
         self.launcher_path = os.path.join(self.path, "launcher")
 
     def __repr__(self):
-        return "RenpyInstance(version={}, path='{}', launcher_path='{}')".format(self.version, self.path, self.launcher_path)  # noqa: E501
+        return "RenpyInstance(version={}, path='{}', launcher_path='{}')".format(
+            self.version, self.path, self.launcher_path
+        )
 
 
 class RenpyRelease(ComparableVersion):
-
     def __init__(self, version=None, url=None):
         super(RenpyRelease, self).__init__(version)
         self.url = url
@@ -97,8 +107,7 @@ class RenpyRelease(ComparableVersion):
         return "RenpyRelease(version={}, url='{}')".format(self.version, self.url)
 
 
-class Registry():
-
+class Registry:
     def __init__(self, filename):
         self.filename = filename
         self.instances = []
@@ -169,10 +178,6 @@ class Registry():
         return iter(self.instances)
 
 
-CACHE = os.path.join(os.path.expanduser("~"), ".renutil")
-REGISTRY = Registry(os.path.join(CACHE, "index.bin"))
-
-
 @contextmanager
 def cd(dir):
     prevdir = os.getcwd()
@@ -210,10 +215,14 @@ def scan_instances(path):
 
 def assure_state():
     if not os.path.isdir(CACHE):
-        logger.debug("Cache directory does not exist, creating it:\n{}".format(CACHE))
+        logger.debug("Cache directory does not exist, creating it: '{}'".format(CACHE))
         os.mkdir(CACHE)
     if not os.access(CACHE, os.R_OK | os.W_OK):
-        logger.debug("Cache directory is not writeable:\n{}\nPlease make sure this script has permission to write to this directory.".format(CACHE))  # noqa: E501
+        logger.debug(
+            "Cache directory is not writeable: '{}'\nPlease make sure this script has permission to write to this directory.".format(
+                CACHE
+            )
+        )
         sys.exit(1)
     instances = scan_instances(CACHE)
     REGISTRY.clear()
@@ -227,7 +236,6 @@ def valid_version(version):
             version = Version(version)
         except ValueError:
             return False
-    # registry = get_registry()
     for instance in REGISTRY:
         if instance.version == version:
             return True
@@ -243,9 +251,13 @@ def get_available_versions(args=None, unknown=None):
     releases = []
     try:
         r = requests.get("https://www.renpy.org/dl/")
-    except:  # noqa: E722
-        logger.error("Could not retrieve version list: No connection could be established.")
-        logger.error("This might mean that you are not connected to the internet or that renpy.org is down.")
+    except Exception:
+        logger.exception(
+            "Could not retrieve version list: No connection could be established."
+        )
+        logger.exception(
+            "This might mean that you are not connected to the internet or that renpy.org is down."
+        )
         sys.exit(1)
     tree = html.fromstring(r.content)
     links = tree.xpath("//a/text()")
@@ -269,7 +281,14 @@ def get_installed_versions(args=None, unknown=None):
 
 @click.group(cls=AliasedGroup)
 @click.option("-d", "--debug", is_flag=True)
-def cli(debug):
+@click.option(
+    "-r",
+    "--registry",
+    default=None,
+    type=click.Path(file_okay=False, resolve_path=True, writable=True),
+    help="The path to store Ren'Py instances in",
+)
+def cli(debug, registry):
     """Commands can be abbreviated by the shortest unique string.
 
     \b
@@ -278,17 +297,38 @@ def cli(debug):
         la -> launch
         li -> list
     """
-    logzero.loglevel(logging.DEBUG if debug else logging.INFO)
+    global CACHE, REGISTRY
+
+    logger.setLevel(logging.DEBUG if debug else logging.INFO)
+
+    if registry:
+        os.makedirs(registry, exist_ok=True)
+        CACHE = registry
+    else:
+        CACHE = os.path.join(os.path.expanduser("~"), ".renutil")
+    REGISTRY = Registry(os.path.join(CACHE, "index.bin"))
+
+    logger.debug("Registry Location: '{}'".format(CACHE))
 
 
 @cli.command()
-@click.option("-a/-l", "--all/--local", "show_all", default=False,
-              help="Show all versions available to download or just the local ones")
-@click.option("-n", "--num-versions", "count", default=5, type=int,
-              help="Amount of versions to show, sorted in descending order")
+@click.option(
+    "-a/-l",
+    "--all/--local",
+    "show_all",
+    default=False,
+    help="Show all versions available to download or just the local ones",
+)
+@click.option(
+    "-n",
+    "--num-versions",
+    "count",
+    default=5,
+    type=int,
+    help="Amount of versions to show, sorted in descending order",
+)
 def list(show_all, count):
-    """List all available versions of Ren'Py.
-    """
+    """List all available versions of Ren'Py."""
     assure_state()
     if show_all:
         releases = get_available_versions()
@@ -296,14 +336,36 @@ def list(show_all, count):
             logger.warning("No releases are available online.")
         else:
             for release in releases[:count]:
-                click.echo(release.version)
+                print(release.version)
     else:
         instances = get_installed_versions()
         if not instances:
             logger.warning("No instances are currently installed.")
         else:
             for release in instances[:count]:
-                click.echo(release.version)
+                print(release.version)
+
+
+@cli.command()
+@click.argument("version", required=True, type=str)
+def show(version):
+    """Show detailed information about an installed version of Ren'Py."""
+    assure_state()
+    if not valid_version(version):
+        logger.error("Invalid version specifier!")
+        sys.exit(1)
+    if not REGISTRY.installed(version):
+        logger.error("Version [salmon]{}[/salmon] is not installed!".format(version))
+        sys.exit(1)
+    instance = REGISTRY.get_instance(version)
+    print("Version: [green]{}[/green]".format(version))
+    print("Install Location: {}".format(os.path.join(CACHE, instance.path)))
+    sdk_filename = "renpy-{}-sdk.zip".format(version)
+    rapt_filename = "renpy-{}-rapt.zip".format(version)
+    SDK_URL = "https://www.renpy.org/dl/{}/{}".format(version, sdk_filename)
+    RAPT_URL = "https://www.renpy.org/dl/{}/{}".format(version, rapt_filename)
+    print("SDK Url: {}".format(SDK_URL))
+    print("RAPT Url: {}".format(RAPT_URL))
 
 
 def download(url, dest):
@@ -319,10 +381,15 @@ def download(url, dest):
     if first_byte >= file_size:
         return
     header = {"Range": "bytes={}-{}".format(first_byte, file_size)}
-    progress_bar = tqdm(total=file_size, initial=first_byte, unit="B",
-                        unit_scale=True, desc=url.split("/")[-1])
+    progress_bar = tqdm(
+        total=file_size,
+        initial=first_byte,
+        unit="B",
+        unit_scale=True,
+        desc=url.split("/")[-1],
+    )
     req = requests.get(url, headers=header, stream=True)
-    with(open(dest, "ab")) as f:
+    with open(dest, "ab") as f:
         for chunk in req.iter_content(chunk_size=1024):
             if chunk:
                 f.write(chunk)
@@ -391,8 +458,7 @@ def patch_file(file, target_line, patch, reverse=False):
 @click.argument("version", required=True, type=str)
 @click.option("-f", "--force", is_flag=True)
 def install(version, force):
-    """Install the specified version of Ren'Py (including RAPT).
-    """
+    """Install the specified version of Ren'Py (including RAPT)."""
     assure_state()
     if REGISTRY.installed(version):
         if force:
@@ -402,8 +468,8 @@ def install(version, force):
             shutil.rmtree(os.path.join(CACHE, instance.path))
             logger.info("Done uninstalling")
         else:
-            logger.error("{} is already installed!".format(version))
-            sys.exit(1)
+            logger.warning("{} is already installed!".format(version))
+            sys.exit(0)
     if not valid_version(version):
         logger.error("Invalid version specifier!")
         sys.exit(1)
@@ -421,13 +487,17 @@ def install(version, force):
     logger.info("Extracting files...")
     sdk_zip = ZipFile(os.path.join(CACHE, sdk_filename), "r")
     rapt_zip = ZipFile(os.path.join(CACHE, rapt_filename), "r")
-    sdk_zip.extractall(path=os.path.join(CACHE, version), members=get_members_zip(sdk_zip))
-    rapt_zip.extractall(path=os.path.join(CACHE, version, "rapt"), members=get_members_zip(rapt_zip))
+    sdk_zip.extractall(
+        path=os.path.join(CACHE, version), members=get_members_zip(sdk_zip)
+    )
+    rapt_zip.extractall(
+        path=os.path.join(CACHE, version, "rapt"), members=get_members_zip(rapt_zip)
+    )
 
     logger.info("Installing RAPT...")
     rapt_path = os.path.join(CACHE, version, "rapt")
 
-    arch = get_platform()
+    arch = get_platform(version)
     if arch == "windows-i686":
         python_path = os.path.join(CACHE, version, "lib", arch, "python.exe")
     else:
@@ -436,21 +506,33 @@ def install(version, force):
     site_package_path = os.path.join(CACHE, version, "lib", arch, "lib", "python2.7")
 
     with cd(rapt_path):
-        patch = "sys.path.insert(0, '{}')\n\nimport ssl\nssl._create_default_https_context = ssl._create_unverified_context\n".format(site_package_path)  # noqa: E501
+        patch = "sys.path.insert(0, '{}')\n\nimport ssl\nssl._create_default_https_context = ssl._create_unverified_context\n".format(
+            site_package_path
+        )
         patch_file("android.py", "import sys", patch, reverse=True)
 
-        patch_file(os.path.join("buildlib", "rapt", "interface.py"),
-                   "def yesno_choice(self, prompt, default=None):",
-                   "        return True\n")
+        patch_file(
+            os.path.join("buildlib", "rapt", "interface.py"),
+            "def yesno_choice(self, prompt, default=None):",
+            "        return True\n",
+        )
 
-        patch_file(os.path.join("buildlib", "rapt", "interface.py"),
-                   "def input(self, prompt, empty=None):",
-                   "        return \"renutil\"\n")
+        patch_file(
+            os.path.join("buildlib", "rapt", "interface.py"),
+            "def input(self, prompt, empty=None):",
+            '        return "renutil"\n',
+        )
 
     os.environ["RAPT_NO_TERMS"] = "no"
     with cd(rapt_path):
-        logger.debug("Running {}".format(" ".join((python_path, "-O", "android.py", "installsdk"))))
-        install = Popen([python_path, "-O", "android.py", "installsdk"], stdout=PIPE, stderr=STDOUT)
+        logger.debug(
+            "Running '{}'".format(
+                " ".join((python_path, "-O", "android.py", "installsdk"))
+            )
+        )
+        install = Popen(
+            [python_path, "-O", "android.py", "installsdk"], stdout=PIPE, stderr=STDOUT
+        )
         for line in install.stdout:
             line = line.strip()
             if line:
@@ -463,31 +545,39 @@ def install(version, force):
 
     head, _ = os.path.split(get_libraries(instance)[0])
     if arch != "windows-i686":
-        paths = [os.path.join(head, "python"), os.path.join(head, "pythonw"),
-                 os.path.join(head, "renpy"), os.path.join(head, "zsync"),
-                 os.path.join(head, "zsyncmake"),
-                 os.path.join(CACHE, instance.rapt_path, "project", "gradlew")]
+        paths = [
+            os.path.join(head, "python"),
+            os.path.join(head, "pythonw"),
+            os.path.join(head, "renpy"),
+            os.path.join(head, "zsync"),
+            os.path.join(head, "zsyncmake"),
+            os.path.join(CACHE, instance.rapt_path, "prototype", "gradlew"),
+            os.path.join(CACHE, instance.rapt_path, "project", "gradlew"),
+        ]
         for path in paths:
             os.chmod(path, S_IRUSR | S_IXUSR)
 
-    with open(os.path.join(CACHE, instance.rapt_path, "project", "gradle.properties"), "r") as f:
-        original_content = f.readlines()
+    for path in (
+        os.path.join(CACHE, instance.rapt_path, "prototype", "gradle.properties"),
+        os.path.join(CACHE, instance.rapt_path, "project", "gradle.properties"),
+    ):
+        with open(path, "r") as f:
+            original_content = f.readlines()
 
-    with open(os.path.join(CACHE, instance.rapt_path, "project", "gradle.properties"), "w") as f:
-        for line in original_content:
-            if line.startswith("org.gradle.jvmargs"):
-                f.write("org.gradle.jvmargs=-Xmx8g\n")
-            else:
-                f.write(line)
+        with open(path, "w") as f:
+            for line in original_content:
+                if line.startswith("org.gradle.jvmargs"):
+                    f.write("org.gradle.jvmargs=-Xmx8g\n")
+                else:
+                    f.write(line)
 
-    logger.info("Done installing {}".format(version))
+    logger.info("Done installing [green]{}[/green]".format(version))
 
 
 @cli.command()
 @click.argument("version", required=True, type=str)
 def uninstall(version):
-    """Uninstall the specified Ren'Py version.
-    """
+    """Uninstall the specified Ren'Py version."""
     assure_state()
     if not REGISTRY.installed(version):
         logger.error("{} is not installed!".format(version))
@@ -497,12 +587,18 @@ def uninstall(version):
     shutil.rmtree(os.path.join(CACHE, instance.path))
 
 
-def get_platform():
-    logger.debug("System: {}".format(platform.system()))
-    logger.debug("Machine: {}".format(platform.machine()))
+def get_platform(version):
+    if isinstance(version, str):
+        version = ComparableVersion(version)
+
+    logger.debug("System: '{}'".format(platform.system()))
+    logger.debug("Machine: '{}'".format(platform.machine()))
 
     if "Darwin" in platform.system():
-        return "darwin-x86_64"
+        if version >= ComparableVersion("7.4.0"):
+            return "mac-x86_64"
+        else:
+            return "darwin-x86_64"
     elif "Windows" in platform.system():
         return "windows-i686"
     elif "x86_64" in platform.machine() or "amd64" in platform.machine():
@@ -518,9 +614,18 @@ def get_libraries(instance):
     root1 = root
     root2 = root
     lib = None
-    arch = get_platform()
+    arch = get_platform(instance)
+    version = str(instance.version).split('.')
+    major = int(version[0])
+    minor = int(version[1])
+    prefix = ""
 
-    if arch == "darwin-x86_64":
+    if major >= 8:
+      prefix = "py3-"
+    elif major == 7 and minor >= 5:
+      prefix = "py2-"
+
+    if arch == "darwin-x86_64" or arch == "mac-x86_64":
         root1 = root + "/../Resources/autorun"
         root2 = root + "/../../.."
     elif arch == "windows-i686":
@@ -540,19 +645,25 @@ def get_libraries(instance):
         sys.exit(1)
 
     for folder in [root, root1, root2]:
-        lib = os.path.join(CACHE, folder, "lib", arch)
+        lib = os.path.join(CACHE, folder, "lib", prefix+arch)
         if os.path.isdir(lib):
             break
     if arch == "windows-i686":
-        lib = os.path.join(lib, "renpy.exe")
+        lib = os.path.join(lib, "python.exe")
     else:
-        lib = os.path.join(lib, "renpy")
+        lib = os.path.join(lib, "python")
 
     if not lib:
-        logger.error("Ren'Py platform files not found in '{}'".format(os.path.join(root, "lib", arch)))
+        logger.error(
+            "Ren'Py platform files not found in '{}'".format(
+                os.path.join(root, "lib", arch)
+            )
+        )
 
     if "LD_LIBRARY_PATH" in os.environ and len(os.environ["LD_LIBRARY_PATH"]) != 0:
-        os.environ["LD_LIBRARY_PATH"] = "{}:{}".format(lib, os.environ["LD_LIBRARY_PATH"])
+        os.environ["LD_LIBRARY_PATH"] = "{}:{}".format(
+            lib, os.environ["LD_LIBRARY_PATH"]
+        )
 
     for folder in [root, root1, root2]:
         base_file = os.path.join(CACHE, folder, "renpy.py")
@@ -565,8 +676,9 @@ def get_libraries(instance):
 @cli.command(context_settings=dict(ignore_unknown_options=True))
 @click.argument("version", required=True, type=str)
 @click.option("-d", "--direct", is_flag=True)
+@click.option("-h", "--headless", is_flag=True)
 @click.argument("args", nargs=-1, type=click.UNPROCESSED)
-def launch(version, direct, args):
+def launch(version, direct, headless, args):
     """Launch the specified version of Ren'Py.
 
     If invoked with default arguments, starts the 'launcher' project,
@@ -596,7 +708,9 @@ def launch(version, direct, args):
         logger.error("{} is not installed!".format(version))
         sys.exit(1)
     instance = REGISTRY.get_instance(version)
-    os.environ["SDL_AUDIODRIVER"] = "dummy"
+    if headless:
+        os.environ["SDL_AUDIODRIVER"] = "dummy"
+        os.environ["SDL_VIDEODRIVER"] = "dummy"
     cmd = get_libraries(instance)
     if not direct:
         cmd += [os.path.join(CACHE, instance.launcher_path)]
@@ -606,27 +720,33 @@ def launch(version, direct, args):
         run(cmd)
     except KeyboardInterrupt:
         assure_state()
-    del os.environ["SDL_AUDIODRIVER"]
+    if headless:
+        del os.environ["SDL_AUDIODRIVER"]
+        del os.environ["SDL_VIDEODRIVER"]
 
 
 @cli.command()
 @click.argument("version", required=True, type=str)
 def cleanup(version):
-    """Clean temporary files of the specified Ren'Py version.
-    """
+    """Clean temporary files of the specified Ren'Py version."""
     assure_state()
     if not REGISTRY.installed(version):
-        logger.error("{} is not installed!".format(version))
+        logger.error("[salmon]{}[/salmon] is not installed!".format(version))
         sys.exit(1)
     instance = REGISTRY.get_instance(version)
-    paths = [os.path.join(instance.path, "tmp"),
-             os.path.join(instance.rapt_path, "assets"),
-             os.path.join(instance.rapt_path, "bin"),
-             os.path.join(instance.rapt_path, "project", "app")]
+    paths = [
+        os.path.join(instance.path, "tmp"),
+        os.path.join(instance.rapt_path, "assets"),
+        os.path.join(instance.rapt_path, "bin"),
+        os.path.join(instance.rapt_path, "project", "app", "build"),
+        os.path.join(instance.rapt_path, "project", "app", "src", "main", "assets"),
+    ]
     for path in paths:
-        if os.path.isdir(os.path.join(CACHE, path)):
-            shutil.rmtree(os.path.join(CACHE, path))
+        full_path = os.path.join(CACHE, path)
+        if os.path.isdir(full_path):
+            logger.info("Cleaning {}".format(full_path))
+            shutil.rmtree(full_path)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     cli()
